/**
 * Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.web.portlet.mvc;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletSession;

import org.apache.commons.lang.Validate;
import org.jasig.web.servlet.mvc.AjaxResponseController;
import org.jasig.web.util.DefaultModelPasser;
import org.jasig.web.util.DefaultSessionKeyGenerator;
import org.jasig.web.util.ModelPasser;
import org.jasig.web.util.SessionKeyGenerator;
import org.springframework.web.portlet.mvc.AbstractController;

/**
 * Base class for AJAX portlet callbacks that handles passing the model to the ajax response
 * servlet via a unique random attribute.
 * 
 * @author Eric Dalquist
 * @version $Revision$
 */
public abstract class AbstractAjaxController extends AbstractController {
    public static final String DEFAULT_AJAX_SERVLET_NAME = "ajaxResponse";
    public static final String DEFAULT_AJAX_HANDLER_ERROR_KEY = "AJAX_HANDLER_ERROR";

    private SessionKeyGenerator sessionKeyGenerator = new DefaultSessionKeyGenerator();
    private ModelPasser modelPasser = new DefaultModelPasser();
    private String ajaxServletName = DEFAULT_AJAX_SERVLET_NAME;
    private String ajaxHandlerErrorKey = DEFAULT_AJAX_HANDLER_ERROR_KEY;
    
    
    public SessionKeyGenerator getSessionKeyGenerator() {
        return this.sessionKeyGenerator;
    }
    /**
     * Used to generate unique keys within a portlet session. Defaults to {@link DefaultSessionKeyGenerator}
     */
    public void setSessionKeyGenerator(SessionKeyGenerator sessionKeyGenerator) {
        Validate.notNull(sessionKeyGenerator);
        this.sessionKeyGenerator = sessionKeyGenerator;
    }
    
    public ModelPasser getModelPasser() {
        return this.modelPasser;
    }
    /**
     * Used to pass the model data to the rendering servlet. Defaults to {@link DefaultModelPasser}
     */
    public void setModelPasser(ModelPasser modelPasser) {
        Validate.notNull(modelPasser);
        this.modelPasser = modelPasser;
    }
    
    public String getAjaxHandlerErrorKey() {
        return this.ajaxHandlerErrorKey;
    }
    /**
     * Model attribute used to pass an exception object to the rendering serlvet if one occurs
     * from calling {@link #handleRenderRequestInternal(javax.portlet.RenderRequest, javax.portlet.RenderResponse)}.
     * Defaults to the value of {@link #DEFAULT_AJAX_HANDLER_ERROR_KEY}
     */
    public void setAjaxHandlerErrorKey(String ajaxHandlerErrorKey) {
        Validate.notNull(ajaxHandlerErrorKey);
        this.ajaxHandlerErrorKey = ajaxHandlerErrorKey;
    }
    
    public String getAjaxServletName() {
        return this.ajaxServletName;
    }
    /**
     * The path to the rendering serlvet do redirect the request to after the model data has been passed.
     * Defaults to the value of {@link #DEFAULT_AJAX_SERVLET_NAME} 
     */
    public void setAjaxServletName(String ajaxServletName) {
        Validate.notNull(ajaxServletName);
        this.ajaxServletName = ajaxServletName;
    }
    
    
    /* (non-Javadoc)
     * @see org.springframework.web.portlet.mvc.AbstractController#handleActionRequestInternal(javax.portlet.ActionRequest, javax.portlet.ActionResponse)
     */
    @Override
    protected final void handleActionRequestInternal(ActionRequest request, ActionResponse response) throws Exception {
        Map<Object, Object> model;
        try {
            model = this.handleAjaxRequestInternal(request, response);
        }
        catch (Exception e) {
            model = new HashMap<Object, Object>();
            this.logger.warn("An exception occurred during handleAjaxRequestInternal()", e);
            model.put(this.ajaxHandlerErrorKey, e);
            //Ensure there is an attribute with the default error key so the servlet can look for an error condition
            if (!DEFAULT_AJAX_HANDLER_ERROR_KEY.equals(this.ajaxHandlerErrorKey)) {
                model.put(DEFAULT_AJAX_HANDLER_ERROR_KEY, true);
            }
        }

        //Generate the session-unique key
        final PortletSession session = request.getPortletSession();
        final String sessionKey = this.sessionKeyGenerator.getNextSessionKey(session);
        
        this.modelPasser.passModelToServlet(request, response, sessionKey, model);
        
        //Get the serlvet url and redirect
        final String ajaxServletUrl = this.getAjaxServletUrl(request, response, sessionKey);
        final String encodedAjaxServletUrl = response.encodeURL(ajaxServletUrl);
        
        if (this.logger.isDebugEnabled()) {
            this.logger.debug("Redirecting ActionRequest to: " + encodedAjaxServletUrl);
        }
        response.sendRedirect(encodedAjaxServletUrl);
    }
    
    /**
     * Generate the servlet URL to redirect the AJAX request to for rendering the response model. The
     * URL will be encoded via {@link PortletResponse#encodeURL(String)} before the redirect is sent.
     * 
     * @param sessionKey The session key to provide on the URL
     */
    protected String getAjaxServletUrl(ActionRequest request, ActionResponse response, String sessionKey) throws UnsupportedEncodingException {
        final String contextPath = request.getContextPath();
        final String characterEncoding = request.getCharacterEncoding();
        final String encodedKeyParam = URLEncoder.encode(AjaxResponseController.SESSION_KEY, characterEncoding);
        final String encodedSessionKey = URLEncoder.encode(sessionKey, characterEncoding);

        return contextPath + "/" + this.ajaxServletName + "?" + encodedKeyParam + "=" + encodedSessionKey;
    }

    /**
     * Called by {@link #handleActionRequestInternal(ActionRequest, ActionResponse)} to handle the ajax callback.
     * The returned model will be passed to the {@link AjaxResponseController} via the {@link ModelPasser#passModelToServlet(ActionRequest, ActionResponse, String, Map)}
     * method.
     * 
     * @see AbstractAjaxController#handleActionRequestInternal(ActionRequest, ActionResponse)
     */
    protected abstract Map<Object, Object> handleAjaxRequestInternal(ActionRequest request, ActionResponse response) throws Exception;
}
